@@ -17,12 +17,14 @@
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  * 
- * $Id: maaP.h,v 1.17 2002/08/02 19:43:15 faith Exp $
+ * $Id: maaP.h,v 1.22 2003/10/26 13:03:24 cheusov Exp $
  */
 
 #ifndef _MAAP_H_
 #define _MAAP_H_
 
+#include <stddef.h>
+
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
@@ -32,18 +34,14 @@
 #endif
 
 /* AIX requires this to be the first thing in the file.  */
-#ifdef __GNUC__
-# define alloca __builtin_alloca
+#if HAVE_ALLOCA_H
+# include <alloca.h>
 #else
-# if HAVE_ALLOCA_H
-#  include <alloca.h>
+# ifdef _AIX
+#  pragma alloca
 # else
-#  ifdef _AIX
- #pragma alloca
-#  else
-#   ifndef alloca /* predefined by HP cc +Olibcalls */
-char *alloca ();
-#   endif
+#  ifndef alloca /* predefined by HP cc +Olibcalls */
+    char *alloca ();
 #  endif
 # endif
 #endif
@@ -65,6 +63,11 @@ char *alloca ();
 # endif
 #endif
 
+#if !HAVE_STRLCPY
+extern size_t strlcpy_libmaa (char *s, const char * wc, size_t size);
+#define strlcpy strlcpy_libmaa
+#endif
+
 #if !HAVE_STRDUP
 extern char *strdup( const char * );
 #endif
@@ -77,6 +80,10 @@ extern long strtol( const char *, char **, int );
 extern unsigned long int strtoul( const char *, char **, int );
 #endif
 
+#if !SIZEOF_VOID_P
+# define SIZEOF_VOID_P sizeof (void *)
+#endif
+
 /* Get time functions */
 #if TIME_WITH_SYS_TIME
 # include <sys/time.h>
@@ -101,14 +108,12 @@ extern unsigned long int strtoul( const char *, char **, int );
 #if HAVE_GETOPT_H
 # include <getopt.h>
 #else
-#if !defined(__FreeBSD__) && !defined(__DGUX__) && !defined(__hpux__)
-#if !defined(__bsdi__) && !defined(__OpenBSD__)
-#if !(defined(__sparc) && !defined(__svr4__))
-extern int  getopt( int, char **, char * );
+#if !HAVE_GETOPT
+extern int  getopt( int, char * const *, const char * );
 extern int  optind;
 extern char *optarg;
-#endif
-#endif
+#else
+# include <unistd.h>
 #endif
 #endif
 
