@@ -1,13 +1,20 @@
-#include <wchar.h>
-#include <assert.h>
-
 /*
   partial mbrlen implementation:
   - doesn't change *ps
   - ignores n
   - s should not be NULL
   - tests first char only
- */
+*/
+
+#include "dictP.h"
+
+#if HAVE_WCHAR_H
+#include <wchar.h>
+#endif
+
+#include <assert.h>
+#include <stddef.h>
+
 size_t mbrlen (const char *s, size_t n, mbstate_t *ps)
 {
    int c;
@@ -18,6 +25,8 @@ size_t mbrlen (const char *s, size_t n, mbstate_t *ps)
 
    if (c == 0)
       return 0;
+   else if (MB_CUR_MAX == 1)
+      return 1;
    else if (c <= 0x7F)
       return 1;
    else if (c <= 0xBF)
