@@ -1,6 +1,3 @@
-#include <wchar.h>
-#include <assert.h>
-
 /*
   partial mbrtowc implementation:
   - doesn't change *ps
@@ -9,6 +6,14 @@
   - pwc should NOT be NULL
 */
 
+#include "dictP.h"
+
+#if HAVE_WCHAR_H
+#include <wchar.h>
+#endif
+
+#include <assert.h>
+
 static const char * utf8_to_ucs4 (
    const char *ptr, wchar_t *result)
 {
@@ -23,7 +28,8 @@ static const char * utf8_to_ucs4 (
    ch = (unsigned char) *ptr++;
 
    if ((ch & 0x80) == 0x00){
-      *result = ch;
+      if (result)
+	 *result = ch;
    }else{
       if ((ch & 0xE0) == 0xC0){
 	 octet_count = 2;
@@ -57,7 +63,8 @@ static const char * utf8_to_ucs4 (
 	 ret |= ((ch & 0x3F) << bits_count);
       }
 
-      *result = ret;
+      if (result)
+	 *result = ret;
    }
 
    return ptr;
@@ -69,6 +76,7 @@ size_t mbrtowc (wchar_t *pwc, const char *s, size_t n, mbstate_t *ps)
 
    assert (s);
    assert (pwc);
+   assert (MB_CUR_MAX > 1);
 
    end = utf8_to_ucs4 (s, pwc);
    if (end)
